// Heatbugs application. Copyright  1996-2000 Swarm Development Group.
// This library is distributed without any warranty; without even the
// implied warranty of merchantability or fitness for a particular purpose.
// See file LICENSE for details and terms of copying.

#import "BatchSwarm.h"
#import "ModelSwarm.h"
#import <simtools.h> // ObjectLoader

#include <math.h>
#include <string.h>
#include <misc.h> // printf

@implementation BatchSwarm

// createBegin: here we set up the default observation parameters.

+ createBegin:  aZone 
{
  BatchSwarm *obj;

  // Superclass createBegin to allocate ourselves.
  obj = [super createBegin: aZone];

  // Fill in the relevant parameters.
  obj->generations = 250;
  obj->logIn = 1;


  return obj;
}

- buildObjects
{
  BOOL graph;

  graph=NO;
  [super buildObjects];

  // IMPORTANT!!

  // Create the model inside us - no longer create `Zone's explicitly.
  // The Zone is now created implicitly through the call to create the
  // `Swarm' inside `self'.

  // But since we don't have any graphics, we load the object from the
  // global `lispAppArchiver' instance which is created automatically
  // from the file called `heatbugs.scm'

  // `modelSwarm' is the key in `heatbugs.scm' which contains the
  // instance variables for the HeatbugModelSwarm class, such as
  // numBugs etc.

  if ((game = [lispAppArchiver getWithZone: self 
                                            key: "game"]) == nil)
    raiseEvent(InvalidOperation, 
               "Can't find the parameters to create game");

  if ((modelSwarm = [lispAppArchiver getWithZone: self 
                                            key: "modelSwarm"]) == nil)
    raiseEvent(InvalidOperation, 
               "Can't find the parameters to create modelSwarm");

  // Now, let the model swarm build its objects.
  [modelSwarm setGame:game];
  [modelSwarm buildObjects];
  experimentDuration=[[modelSwarm getCounter] getAValue]*generations;
  loggingFrequency=[[modelSwarm getCounter] getAValue];


  if(loggingFrequency)
    { 
      groups=[modelSwarm getGroups];
      [groups toFile: "pop.dat"];
      
#include "aGraph.ch.m"
#include "bGraph.ch.m"
    }
  // All done - we're ready to build a schedule and go.

  return self;
}  

// Create the actions necessary for the simulation. This is where
// the schedule is built (but not run!)

- buildActions 
{
  [super buildActions];
  
  // First, let our model swarm build its own schedule.

  [modelSwarm buildActions];
  
  if(loggingFrequency)
    {
      
      // Create an ActionGroup for display. This is pretty minimal in this
      // case. Note, there's no doTkEvents message - no control panel!
      
      displayActions = [ActionGroup create: self];
      
      // Now schedule the update of the unhappyGraph, which will in turn 
      // cause the fileI/O to occur...
      
       [displayActions createActionTo: groups message: M(printGroups)];
       [displayActions createActionTo: aGraph message: M(step)];
       [displayActions createActionTo: bGraph message: M(step)];
      
      // the displaySchedule controls how often we write data out.
      displaySchedule = [Schedule createBegin: self];
      [displaySchedule setRepeatInterval: loggingFrequency];
      displaySchedule = [displaySchedule createEnd];
      
      [displaySchedule at: 0 createAction: displayActions];
    }
  
  // We also add in a "stopSchedule", another schedule with an absolute
  // time event - stop the system at time . 
  
  stopSchedule = [Schedule create: self];
  [stopSchedule at: experimentDuration 
                createActionTo: self 
                message: M(stopRunning)];
  
  return self;
}  

// activateIn: - get the Swarm ready to run.
- activateIn:  swarmContext 
{
  // First, activate ourselves (just pass along the context).
  [super activateIn: swarmContext];
  
  // We need to activate the model swarm.
  [modelSwarm activateIn: self];
  
  // Now activate our schedules in ourselves. Note that we just activate
  // both schedules: the activity library will merge them properly.
  [stopSchedule activateIn: self];
  if (loggingFrequency)
    [displaySchedule activateIn: self];
  
  // Activate returns the swarm activity - the thing that's ready to run.
  return [self getActivity];
}

// the HeatbugObserverSwarm had a go method inherited from GUISwarm,
// but we have to define our own here. It's pretty simple. There's also
// a friendly message printed out here just in case someone is confused
// when they run heatbugs and see no graphics.

- go 
{
  printf ("You typed `foo -b' or `foo --batch', so we're running without graphics.\n");

  printf ("foo is running for %d generation.\n",generations) ;
  printf ("of %d rounds each\n",            loggingFrequency);
 
  if (loggingFrequency)
    printf ("It is logging data every %d timesteps",
            loggingFrequency);
  printf("\n");
  [[self getActivity] run];
  return [[self getActivity] getStatus];
}

// And the termination method. When this fires we just terminate everything
// that's running and close our output file(s) by dropping the EZGraph which
// "owns" the sequence(s) we are logging.

- stopRunning 
{
  [getTopLevelActivity() terminate]; // Terminate the simulation.

  
  return self;
}

@end
